@@ -17,6 +17,7 @@ where
 import Control.Monad (MonadPlus (mplus), foldM, foldM_)
 import Control.Monad.State (MonadTrans (lift))
 import Data.List (intersperse, sortOn)
+import Data.Foldable (toList)
 import qualified Data.Map as M
 import qualified Data.Map.Ordered as OM
 import Data.Maybe (isJust, fromMaybe)
@@ -34,7 +35,7 @@ import Text.Parsec
 import Typst.Bind (destructuringBind)
 import Typst.Constructors (getConstructor)
 import Typst.Methods (getMethod)
-import Typst.Module.Standard (loadFileText, standardModule, symModule)
+import Typst.Module.Standard (loadFileText, standardModule, symModule, getPath)
 import Typst.Module.Math (mathModule)
 import Typst.MathClass (mathClassOf, MathClass(Relation))
 import Typst.Parse (parseTypst)
@@ -63,7 +64,12 @@ evaluateTypst operations fp =
   runParserT
     (do contents <- mconcat <$> many pContent <* eof
         -- "All documents are automatically wrapped in a document element."
-        pure $ Elt "document" Nothing [("body", VContent contents)])
+        els <- element (Identifier "document")
+                   Arguments {positional = [VContent contents],
+                              named = []}
+        case toList els of
+          [e] -> pure e
+          _ -> fail "element returned something other than a singleton")
     initialEvalState { evalOperations = operations,
                        evalLocalDir = takeDirectory fp }
     fp
@@ -664,26 +670,6 @@ evalExpr expr = applyShowRulesToVal =<<
       case maybeNegate v of
         Nothing -> fail $ "Can't negate " <> show v
         Just v' -> pure v'
-    ToPower e1 e2 -> do
-      e <- evalExpr e1
-      b <- evalExpr e2
-      case (b, e) of
-        (VInteger i, VInteger j) ->
-          pure $
-            VInteger $
-              floor ((fromIntegral i :: Double) ** (fromIntegral j :: Double))
-        (VInteger i, VRatio j) ->
-          pure $
-            VFloat ((fromIntegral i :: Double) ** (fromRational j :: Double))
-        (VRatio i, VInteger j) ->
-          pure $
-            VFloat (fromRational i ** (fromIntegral j :: Double))
-        (VRatio i, VRatio j) -> pure $ VFloat (fromRational i ** fromRational j)
-        (VFloat i, VInteger j) -> pure $ VFloat (i ** (fromIntegral j :: Double))
-        (VFloat i, VFloat j) -> pure $ VFloat (i ** j)
-        (VInteger i, VFloat j) -> pure $ VFloat ((fromIntegral i :: Double) ** j)
-        (VFloat i, VRatio j) -> pure $ VFloat (i ** fromRational j)
-        _ -> fail $ "Can't exponentiate " <> show b <> " to " <> show e
     Plus e1 e2 -> do
       v1 <- evalExpr e1
       v2 <- evalExpr e2
@@ -838,8 +824,9 @@ evalExpr expr = applyShowRulesToVal =<<
       evalExpr e -- TODO for now we just ignore "context"
     Return mbe -> do
       -- these flow directives are examined in CodeBlock
+      result <- maybe (pure VNone) evalExpr mbe
       updateState (\st -> st {evalFlowDirective = FlowReturn (isJust mbe)})
-      maybe (pure VNone) evalExpr mbe
+      pure result
     Continue -> do
       updateState (\st -> st {evalFlowDirective = FlowContinue})
       pure VNone
@@ -875,6 +862,7 @@ evalExpr expr = applyShowRulesToVal =<<
           (cs, _) <- loadModule t
           pure $ VContent cs
         _ -> fail "Include requires a path"
+    Underscore -> fail "expected expression, found underscore"
 
 toFunction ::
   Monad m =>
@@ -909,7 +897,10 @@ toFunction mbname params e = do
                 (x : xs) -> do
                   destructuringBind addIdentifier parts x
                   pure $ as {positional = xs}
-            setParam as SkipParam = pure as
+            setParam as SkipParam = do
+              case positional as of
+                [] -> fail ("missing argument: pattern parameter")
+                (_ : xs) -> pure $ as {positional = xs}
         inBlock FunctionScope $ do
           -- We create a closure around the identifiers defined
           -- where the function is defined:
@@ -1024,7 +1015,8 @@ loadModule modname = do
                     Identifier (T.pack $ takeBaseName fp),
                     Nothing,
                     txt )
-  case parseTypst fp txt of
+  fp' <- getPath fp  -- get full path for element positions
+  case parseTypst fp' txt of
     Left err -> fail $ show err
     Right ms -> do
       operations <- evalOperations <$> getState
