@@ -767,12 +767,19 @@ isSpecial '=' = True
 isSpecial '(' = True -- so we don't gobble ( before URLs
 isSpecial _ = False
 
-pIdentifier :: P Identifier
-pIdentifier = lexeme $ try $ do
+pIdentifierOrUnderscore :: P Identifier
+pIdentifierOrUnderscore = lexeme $ try $ do
   c <- satisfy isIdentStart
   cs <- many $ satisfy isIdentContinue
   pure $ Identifier $ T.pack (c : cs)
 
+pIdentifier :: P Identifier
+pIdentifier = do 
+  ident <- pIdentifierOrUnderscore
+  if ident == "_"
+    then fail "expected identifier, found underscore"
+    else pure $ ident
+
 -- ident_start ::= unicode(XID_Start)
 -- ID_Start characters are derived from the Unicode General_Category of
 -- uppercase letters, lowercase letters, titlecase letters, modifier letters,
@@ -817,23 +824,26 @@ pKeyword t = lexeme $ try $ string t *> notFollowedBy (satisfy isIdentContinue)
 --             "break", "continue", "return", "import", "include", "from"]
 
 pExpr :: P Expr
-pExpr = buildExpressionParser operatorTable pBasicExpr
+pExpr = buildExpressionParser operatorTable (buildExpressionParser basicOperatorTable (pBaseExpr pIdentOrUnderscore))
+--
 
 -- A basic expression excludes the unary and binary operators outside of parens,
--- but includes field access and function application. Needed for pHash.
+-- but includes field access and function application. A single underscore is
+-- not a valid identifier in a basic expression and should be interpreted as markup.
+-- Needed for pHash.
 pBasicExpr :: P Expr
-pBasicExpr = buildExpressionParser basicOperatorTable pBaseExpr
+pBasicExpr = buildExpressionParser basicOperatorTable (pBaseExpr pIdent)
 
 pQualifiedIdentifier :: P Expr
 pQualifiedIdentifier =
   buildExpressionParser (replicate 4 [fieldAccess]) pIdent
 
-pBaseExpr :: P Expr
-pBaseExpr =
+pBaseExpr :: P Expr -> P Expr
+pBaseExpr ident_parser =
   pLiteral
     <|> pKeywordExpr
     <|> pFuncExpr
-    <|> pIdent
+    <|> ident_parser
     <|> pArrayExpr
     <|> pDictExpr
     <|> inParens pExpr
@@ -879,8 +889,7 @@ operatorTable =
   take 12 (cycle [[fieldAccess], [functionCall]])
     ++
     -- precedence 7 (repeated because of parsec's quirks with postfix, prefix)
-    replicate 6 [Postfix (ToPower <$> try (char 'e' *> notFollowedBy letter *> pExpr))]
-    ++ replicate 6 [Prefix (Negated <$ op "-"), Prefix (id <$ op "+")]
+    replicate 6 [Prefix (Negated <$ op "-"), Prefix (id <$ op "+")]
     ++ [
          -- precedence 6
          [ Infix (Times <$ op "*") AssocLeft,
@@ -949,25 +958,36 @@ pNumber = try $ do
         _ -> fail $ "could not read " <> num <> " as octal digits"
     _ -> do
       as <- many1 digit <|> ("0" <$ lookAhead (try (char '.' *> digit)))
-      pe <- option [] $ string "."
+      pe <- 
+        option 
+          [] 
+          (try ( do
+              void $ char '.'
+              notFollowedBy (char '.' <|> satisfy isIdentStart)
+              pure "."
+          ))
       bs <- many digit
       es <-
         option
           ""
           ( do
-              void $ try $ char 'e' *> lookAhead (digit <|> char '-')
-              minus <- option [] $ count 1 (char '-')
+              void $ try $ oneOf "eE" *> lookAhead (digit <|> oneOf "+-")
+              sign <- option [] $ count 1 (oneOf "+-")
               ds <- many1 digit
-              pure ("e" ++ minus ++ ds)
+              pure ("e" ++ sign ++ ds)
           )
       let num = pref ++ as ++ pe ++ bs ++ es
       case readMaybe num of
         Just (i :: Integer) -> pure $ Left i
         Nothing ->
-          case readMaybe num of
+          case readMaybe $ addTrailingZero num of
             Just (d :: Double) -> pure $ Right d
             Nothing -> fail $ "could not read " <> num <> " as integer"
 
+addTrailingZero :: String -> String
+addTrailingZero []  = [] 
+addTrailingZero num@(_:_) = if last num == '.' then num ++ "0" else num
+
 pNumeric :: P Literal
 pNumeric = lexeme $ do
   result <- pNumber
@@ -1001,6 +1021,13 @@ pUnit =
 pIdent :: P Expr
 pIdent = Ident <$> pIdentifier
 
+pIdentOrUnderscore :: P Expr
+pIdentOrUnderscore = do
+  ident <- pIdentifierOrUnderscore
+  if ident == "_"
+    then pure $ Underscore
+    else pure $ Ident ident
+
 pBlock :: P Expr
 pBlock = Block <$> (pCodeBlock <|> pContent)
 
@@ -1075,7 +1102,7 @@ pFuncExpr = try $ FuncExpr <$> pParamsOrIdent <*> (sym "=>" *> pExpr)
   where
     pParamsOrIdent =
       pParams
-        <|> (do i <- pIdentifier
+        <|> (do i <- pIdentifierOrUnderscore
                 if i == "_"
                    then pure [SkipParam]
                    else pure [NormalParam i])
@@ -1135,7 +1162,7 @@ pParam =
           )
     pSkipParam = SkipParam <$ sym "_"
     pNormalOrDefaultParam = do
-      i <- pIdentifier
+      i <- try pIdentifier
       (DefaultParam i <$> (sym ":" *> pExpr)) <|> pure (NormalParam i)
     pDestructuringParam = do
       DestructuringBind parts <- pDestructuringBind
@@ -1149,7 +1176,7 @@ pBasicBind = BasicBind <$> try (pBindIdentifier <|> inParens pBindIdentifier)
 
 pBindIdentifier :: P (Maybe Identifier)
 pBindIdentifier = do
-  ident <- pIdentifier
+  ident <- pIdentifierOrUnderscore
   if ident == "_"
      then pure Nothing
      else pure $ Just ident
