@@ -118,7 +118,6 @@ data Spreadable a =
 data Expr
   = Literal Literal
   | Negated Expr
-  | ToPower Expr Expr
   | Times Expr Expr
   | Divided Expr Expr
   | Plus Expr Expr
@@ -156,6 +155,7 @@ data Expr
   | Label Text
   | Break
   | Continue
+  | Underscore
   deriving (Show, Ord, Eq, Data, Typeable)
 
 data Imports
